
/**
 * 防抖
 * @param {Number} delay 延迟时间
 * @param {*} callback
 */
export const debounce = (delay, callback) => {
  let timer = null
  return function () {
    clearTimeout(timer)
    const [that, args] = [this, arguments]
    timer = setTimeout(() => {
      callback.apply(that, args)
    }, delay)
  }
}

/**
 * 获取两个数组不同元素(差集)
 * @param {Array} arr1
 * @param {Array} arr2
 * @return {Array}
 */
export const getArrDifference = (arr1, arr2) => {
  return arr1.concat(arr2).filter((v, i, arr) => {
    return arr.indexOf(v) === arr.lastIndexOf(v)
  })
}

/**
   类型判断
  判断核心使用Object.prototype.toString，这种方式可以准确的判断数据类型。
 * @param {any} target
 * @param {string} type
 * @return {boolean}
*/
export const isType = (target, type) => {
  let targetType = Object.prototype.toString.call(target).slice(8, -1).toLowerCase()
  return targetType === type.toLowerCase()
}

/**
* 数组元素交换位置
* @param {array} arr 数组
* @param {number} index1 添加项目的位置
* @param {number} index2 删除项目的位置
*/
export const swapArray = (arr, index1, index2) => {
  arr[index1] = arr.splice(index2, 1, arr[index1])[0]
  return arr
}

/**
 * 检查字符串是否为JSON
 * @param {*} str
 */
export const isJSON = str => {
  if (typeof str === 'string') {
    try {
      var obj = JSON.parse(str)
      if (typeof obj === 'object' && obj) {
        return true
      } else {
        return false
      }
    } catch (e) {
      console.log('error：' + str + '!!!' + e)
      return false
    }
  } else {
    return false
  }
}

/**
 * 判断字符串是否是正则表达式
 * @param {*} reg
 */
export const isReg = (reg) => {
  let isReg
  try {
    // eslint-disable-next-line
    isReg = eval(reg) instanceof RegExp
  } catch (e) {
    isReg = false
  }
  return isReg
}

export function timeOut (fn, time = 500) { // 延时器
  let timer = setTimeout(() => {
    fn()
    clearTimeout(timer)
  }, time)
}

/**
* 获取文本px宽度
* @param font{String}: 字体样式 默认："10px sans-serif"
**/
export const strPxWidth = function (text, font) {
  let canvas = document.createElement('canvas')
  let context = canvas.getContext('2d')

  font && (context.font = font)
  let metrics = context.measureText(text)
  return metrics.width
}
